//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 28/04/01; 17:40:54

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "kblib.h"
#include "snaky.h"

//---------------------------------------------------------------------------

extern KBSTATE kb_state;
extern KBSTATE kb_state_DOWN;
extern KBSTATE kb_state_UP;
extern void (*HandlerFuncPtr5)(void);

//---------------------------------------------------------------------------

unsigned char sprite1[] = {0x3C,0x7E,0xFF,0xFF,0xFF,0xFF,0x7E,0x3C};
unsigned char sprite2[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

//---------------------------------------------------------------------------

TPoint Points[SCR_WIDTH*SCR_HEIGHT];
TPoint DestPoint;
TPoint NewPoint;

int PointsCount;
int SnakeWay;
int Score;
int LoseGame;
int GameSpeed=2;


//---------------------------------------------------------------------------

void TILE_ON(int x,int y)
{
	Sprite8_OR(x<<3,y<<3,8,sprite1,LCD_MEM);
}
//---------------------------------------------------------------------------

void TILE_OFF(int x,int y)
{
	Sprite8_AND(x<<3,y<<3,8,sprite2,LCD_MEM);
}
//---------------------------------------------------------------------------

void NewDest(void)
{
	int a;
	TPoint Point;
	int IsOverSnake;
	
	do
	{
		Point.x = random(SCR_WIDTH);
		Point.y = random(SCR_HEIGHT);
		
		
		IsOverSnake = 0;
		
		for (a=0;a<PointsCount;a++)
		if ((Point.x==Points[a].x && Point.y==Points[a].y))
		IsOverSnake = 1;
		
	}
	while (IsOverSnake);
	
	DestPoint = Point;
	
	// DEST TILE ON
	TILE_ON(DestPoint.x,DestPoint.y);
	
}
//---------------------------------------------------------------------------

void NewGame(void)
{
	int a;
	
	ClrScr();
	DrawLine(0,90,159,90,A_NORMAL);
	
	
	PointsCount = 3;
	SnakeWay = TD_RIGHT;
	Score = 0;
	
	for (a=0;a<PointsCount;a++)
	{
		Points[a].x = PointsCount-1-a;
		Points[a].y = 0;
		// SHOW SNAKE TILE
		TILE_ON(Points[a].x,Points[a].y);
	}
	
	NewDest();
	
	WaitForKeyDOWN(KEYFLAGS_2ND);
	WaitForAllKeysReleased();
	
}
//---------------------------------------------------------------------------

void ChangeWay(int way_wanted)
{
	int opp_way=((SnakeWay+2)&3);
	if (way_wanted!=opp_way)
	SnakeWay=way_wanted;
}
//---------------------------------------------------------------------------

void SnakeStep(void)
{
	
	
	NewPoint = Points[0];
	
	switch (SnakeWay)
	{
		case TD_LEFT : NewPoint.x--;break;
		case TD_RIGHT : NewPoint.x++;break;
		case TD_UP : NewPoint.y--;break;
		case TD_DOWN : NewPoint.y++;break;
	}
	
	if (NewPoint.x<0 || NewPoint.x==SCR_WIDTH
	|| NewPoint.y<0 || NewPoint.y==SCR_HEIGHT)
	LoseGame = 1;
	
	int a;
	for (a=0;a<PointsCount;a++)
	if (NewPoint.x==Points[a].x && NewPoint.y==Points[a].y)
	LoseGame = 1;
	
	if (LoseGame)
	return;
	
	
	if (DestPoint.x==NewPoint.x && DestPoint.y==NewPoint.y)
	{
		Score++;
		PointsCount++;
		
		for (a=PointsCount-1;a>0;a--)
		Points[a] = Points[a-1];
		
		Points[0] = NewPoint;
		
		NewDest();
	}
	else
	{
		// LAST TILE OFF
		TILE_OFF(Points[PointsCount-1].x,Points[PointsCount-1].y);
		
		// MOVE POINTS
		for (a=PointsCount-1;a>0;a--)
		Points[a] = Points[a-1];
		
		Points[0] = NewPoint;
		
		// FIRST TILE ON
		TILE_ON(Points[0].x,Points[0].y);
	}
	
	
	
}
//---------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	clrscr();
	FontSetSys(0);
	printf("Snaky 1.2 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	FontSetSys(0);
	randomize();
	
	HandlerFuncPtr5 = NULL;
	KB_Enable();
	WaitForAllKeysReleased();
	
	NewGame();
	int TimeCounter=0;
	
	for (;;)
	{
		
		
		LoseGame = 0;
		
		if (!(TimeCounter++%16))
		SnakeStep();
		
		if (LoseGame)
		{
			DrawStr(70,92,"You lose!",A_REPLACE);
			WaitForAllKeysReleased();
			WaitForKeyDOWN(KEYFLAGS_2ND);
			WaitForAllKeysReleased();
			NewGame();
			continue;
		}
		
		
		// STATS
		char buffer[16];
		sprintf(buffer,"Score : %03d",Score);
		DrawStr(0,92,buffer,A_REPLACE);
		
		
		GetKeyboardState();
		
		if (kb_state&KEYFLAGS_ESC)
		break;
		else if (kb_state&KEYFLAGS_LEFT)
		ChangeWay(TD_LEFT);
		else if (kb_state&KEYFLAGS_RIGHT)
		ChangeWay(TD_RIGHT);
		else if (kb_state&KEYFLAGS_UP)
		ChangeWay(TD_UP);
		else if (kb_state&KEYFLAGS_DOWN)
		ChangeWay(TD_DOWN);
		else if (kb_state&KEYFLAGS_DIAMOND)
		{
			WaitForAllKeysReleased();
			WaitForKeyDOWN(KEYFLAGS_2ND);
			WaitForAllKeysReleased();
		}
		
	}
	
	WaitForAllKeysReleased();
	KB_Disable();
	
	
}
